#include "cgic.h"
#include "auth.h"
#include "config.h"
#include "utils.h"

#define UUID_SIZE 36

/* TODO: place cookie normally */
void
validate_credentials()
{
    char username[RA_USER_MAX_LENGTH + 1], password[RA_PASSWORD_MAX_LENGTH + 1];
    char session_id[UUID_SIZE];
    cgiFormResultType err;
    int auth;

    err = cgiFormString("username", username, RA_USER_MAX_LENGTH + 1);
    if (err == cgiFormTruncated) {
        fprintf(cgiOut, "Username too long(max %d chars)\n", RA_USER_MAX_LENGTH);
        return;
    }
    if (err == cgiFormNotFound) {
        fprintf(cgiOut, "Username not provided\n");
        return;
    }

    err = cgiFormString("password", password, RA_PASSWORD_MAX_LENGTH + 1);
    if (err == cgiFormTruncated) {
        fprintf(cgiOut, "Password too long(max %d+1(NUL) bytes)\n", RA_PASSWORD_MAX_LENGTH);
        return;
    }
    if (err == cgiFormNotFound) {
        fprintf(cgiOut, "Password not provided\n");
        return;
    }

    if (!is_valid_username(username)) {
        fprintf(cgiOut, "Username must be [A-Za-z0-9_]\n");
        return;
    }
    
    if (!is_valid_password(password)) {
        fprintf(cgiOut, "Password must be a sequence of bytes in range 32-255\n");
        return;
    }

    auth = authenticate(username, password, session_id);
    if (auth < 0) {
        fprintf(cgiOut, "Some error occured, contact system administrator\n");
        return;
    }

    if (auth) {
        cgiHeaderCookieSet(SESSION_COOKIE_NAME, session_id, RA_SESSION_EXPIRE, "/", HOSTNAME, 0);
        fprintf(cgiOut, "You've successfully logged in as %s\n", username);
    } else {
        fprintf(cgiOut, "Failed to log in, check credentials\n");
    }
}

void
print_login_form()
{
    fputs("<form action=\"\" method=\"POST\">\n"
        "<label>Username:</label>\n"
        "<input type=\"text\" name=\"username\"><br>\n"
        "<label>Password: </label>\n"
        "<input type=\"password\" name=\"password\"><br>\n"
        "<input type=\"submit\" name=\"login\" value=\"Submit\">\n"
    "</form>\n", cgiOut);
}

int
cgiMain()
{
    cgiHeaderContentType("text/html");

    fprintf(cgiOut, "<html>\n");

    fprintf(cgiOut, "<head>\n");
    fprintf(cgiOut, "<title> Simple discuss powered by qments </title>\n");
    fprintf(cgiOut, "</head>\n");

    fprintf(cgiOut, "<body>\n");

    if (cgiFormSubmitClicked("login") == cgiFormSuccess) {
        validate_credentials();
    }

    print_login_form();

    fprintf(cgiOut, "</body>\n");

    fprintf(cgiOut, "</html>\n");

    return 0;

}
